from sqlalchemy import Column, String, Boolean, ForeignKey, Integer, Float, text
from sqlalchemy.orm import relationship
from sqlalchemy.ext.declarative import declarative_base
from api.v1.models.base_model import BaseTableModel

class Wallet(BaseTableModel):
    __tablename__ = "wallets"

    user_id = Column(ForeignKey("users.id"), nullable=False, unique=True)
    balance = Column(Float, nullable=False, server_default=text("0.0"))

    user = relationship("User", back_populates="wallet")
    tokens = relationship(
        "TokenBalance", back_populates="wallet", cascade="all, delete-orphan"
    )

    def __str__(self):
        return f"Wallet of {self.user.email}"
    


class TokenBalance(BaseTableModel):
    __tablename__ = "token_balances"

    wallet_id = Column(ForeignKey("wallets.id"), nullable=False)
    token_id = Column(ForeignKey("tokens.id"), nullable=False)
    amount = Column(Float, nullable=False, server_default=text("0.0"))

    wallet = relationship("Wallet", back_populates="tokens")
    token = relationship("Token", back_populates="token_balances")

    def __str__(self):
        return f"{self.amount} {self.token.symbol} in {self.wallet.user.email}'s wallet"
